


#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>

#include <libubox/list.h>

#include <easy/easy.h>
#include <wifiutils.h>

#include "timer_impl.h"
#include "wifi_dataelements.h"
#include "common.h"

#ifndef BIT
#define BIT(n)	(1 << (n))
#endif

#if 0
static struct wifi_opclass_entry wifi_opclass_global[] = {
	{ 81, BAND_2, BW20, EXTCH_NONE, {20, 13, {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13}}},
	{ 82, BAND_2, BW20, EXTCH_NONE, {20, 1, {14}}},
	{ 83, BAND_2, BW40, EXTCH_ABOVE, {20, 9, {1, 2, 3, 4, 5, 6, 7, 8, 9}}},
	{ 84, BAND_2, BW40, EXTCH_BELOW, {20, 9, {5, 6, 7, 8, 9, 10, 11, 12, 13}}},

	{ 115, BAND_5, BW20, EXTCH_NONE, {23, 4, {36, 40, 44, 48}}},
	{ 116, BAND_5, BW40, EXTCH_ABOVE, {23, 2, {36, 44}}},
	{ 117, BAND_5, BW40, EXTCH_BELOW, {23, 2, {40, 48}}},
	{ 118, BAND_5, BW20, EXTCH_NONE, {23, 4, {52, 56, 60, 64}}},
	{ 119, BAND_5, BW40, EXTCH_ABOVE, {23, 2, {52, 60}}},
	{ 120, BAND_5, BW40, EXTCH_BELOW, {23, 2, {56, 64}}},
	{ 121, BAND_5, BW20, EXTCH_NONE, {30, 12, {100, 104, 108, 112, 116, 120, 124, \
					       128, 132, 136, 140, 144}}},
	{ 122, BAND_5, BW40, EXTCH_ABOVE, {30, 6, {100, 108, 116, 124, 132, 140}}},
	{ 123, BAND_5, BW40, EXTCH_BELOW, {30, 6, {104, 112, 120, 128, 136, 144}}},
	{ 124, BAND_5, BW20, EXTCH_NONE, {36, 4, {149, 153, 157, 161}}},
	{ 125, BAND_5, BW20, EXTCH_NONE, {36, 6, {149, 153, 157, 161, 165, 169}}},
	{ 126, BAND_5, BW40, EXTCH_ABOVE, {36, 2, {149, 157}}},
	{ 127, BAND_5, BW40, EXTCH_BELOW, {36, 2, {153, 161}}},
	{ 128, BAND_5, BW80, EXTCH_AUTO, {23, 6, {36, 52, 100, 116, 132, 149}}},
	{ 129, BAND_5, BW160, EXTCH_AUTO, {23, 2, {36, 100}}},
	{ 130, BAND_5, BW8080, EXTCH_AUTO, {23, 6, {36, 52, 100, 116, 132, 149}}},
};

#define wifi_opclass_global_size	\
	sizeof(wifi_opclass_global)/sizeof(wifi_opclass_global[0])

#endif

static const struct wifi_radio_opclass e4 = {
	.num_opclass = 20,
	.entry = {
		{
			.id = 81,
			.bandwidth = 20,
			.num_channel = 13,
			.channel = {
				{ .channel = 1, .ctrl_channels = {1}},
				{ .channel = 2, .ctrl_channels = {2}},
				{ .channel = 3, .ctrl_channels = {3}},
				{ .channel = 4, .ctrl_channels = {4}},
				{ .channel = 5, .ctrl_channels = {5}},
				{ .channel = 6, .ctrl_channels = {6}},
				{ .channel = 7, .ctrl_channels = {7}},
				{ .channel = 8, .ctrl_channels = {8}},
				{ .channel = 9, .ctrl_channels = {9}},
				{ .channel = 10, .ctrl_channels = {10}},
				{ .channel = 11, .ctrl_channels = {11}},
				{ .channel = 12, .ctrl_channels = {12}},
				{ .channel = 13, .ctrl_channels = {13}},
			}
		},
		{
			.id = 82,
			.bandwidth = 20,
			.num_channel = 1,
			.channel = {
				{ .channel = 14, .ctrl_channels = {14}},
			}
		},
		{
			.id = 83,
			.bandwidth = 40,
			.num_channel = 9,
			.channel = {
				{ .channel = 1, .ctrl_channels = {1}},
				{ .channel = 2, .ctrl_channels = {2}},
				{ .channel = 3, .ctrl_channels = {3}},
				{ .channel = 4, .ctrl_channels = {4}},
				{ .channel = 5, .ctrl_channels = {5}},
				{ .channel = 6, .ctrl_channels = {6}},
				{ .channel = 7, .ctrl_channels = {7}},
				{ .channel = 8, .ctrl_channels = {8}},
				{ .channel = 9, .ctrl_channels = {9}},
			}
		},
		{
			.id = 84,
			.bandwidth = 40,
			.num_channel = 9,
			.channel = {
				{ .channel = 5, .ctrl_channels = {5}},
				{ .channel = 6, .ctrl_channels = {6}},
				{ .channel = 7, .ctrl_channels = {7}},
				{ .channel = 8, .ctrl_channels = {8}},
				{ .channel = 9, .ctrl_channels = {9}},
				{ .channel = 10, .ctrl_channels = {10}},
				{ .channel = 11, .ctrl_channels = {11}},
				{ .channel = 12, .ctrl_channels = {12}},
				{ .channel = 13, .ctrl_channels = {13}},
			}
		},
		{
			.id = 115,
			.bandwidth = 20,
			.num_channel = 4,
			.channel = {
				{ .channel = 36, .ctrl_channels = {36}},
				{ .channel = 40, .ctrl_channels = {40}},
				{ .channel = 44, .ctrl_channels = {44}},
				{ .channel = 48, .ctrl_channels = {48}},
			}
		},
		{
			.id = 116,
			.bandwidth = 40,
			.num_channel = 2,
			.channel = {
				{ .channel = 36, .ctrl_channels = {36}},
				{ .channel = 44, .ctrl_channels = {44}},
			}
		},
		{
			.id = 117,
			.bandwidth = 40,
			.num_channel = 2,
			.channel = {
				{ .channel = 40, .ctrl_channels = {40}},
				{ .channel = 48, .ctrl_channels = {48}},
			}
		},
		{
			.id = 118,
			.bandwidth = 20,
			.num_channel = 4,
			.channel = {
				{ .channel = 52, .ctrl_channels = {52}},
				{ .channel = 56, .ctrl_channels = {56}},
				{ .channel = 60, .ctrl_channels = {60}},
				{ .channel = 64, .ctrl_channels = {64}},
			}
		},
		{
			.id = 119,
			.bandwidth = 40,
			.num_channel = 2,
			.channel = {
				{ .channel = 52, .ctrl_channels = {52}},
				{ .channel = 60, .ctrl_channels = {60}},
			}
		},
		{
			.id = 120,
			.bandwidth = 40,
			.num_channel = 2,
			.channel = {
				{ .channel = 56, .ctrl_channels = {56}},
				{ .channel = 64, .ctrl_channels = {64}},
			}
		},
		{
			.id = 121,
			.bandwidth = 20,
			.num_channel = 12,
			.channel = {
				{ .channel = 100, .ctrl_channels = {100}},
				{ .channel = 104, .ctrl_channels = {104}},
				{ .channel = 108, .ctrl_channels = {108}},
				{ .channel = 112, .ctrl_channels = {112}},
				{ .channel = 116, .ctrl_channels = {116}},
				{ .channel = 120, .ctrl_channels = {120}},
				{ .channel = 124, .ctrl_channels = {124}},
				{ .channel = 128, .ctrl_channels = {128}},
				{ .channel = 132, .ctrl_channels = {132}},
				{ .channel = 136, .ctrl_channels = {136}},
				{ .channel = 140, .ctrl_channels = {140}},
				{ .channel = 144, .ctrl_channels = {144}},
			}
		},
		{
			.id = 122,
			.bandwidth = 40,
			.num_channel = 6,
			.channel = {
				{ .channel = 100, .ctrl_channels = {100}},
				{ .channel = 108, .ctrl_channels = {108}},
				{ .channel = 116, .ctrl_channels = {116}},
				{ .channel = 124, .ctrl_channels = {124}},
				{ .channel = 132, .ctrl_channels = {132}},
				{ .channel = 140, .ctrl_channels = {140}},
			}
		},
		{
			.id = 123,
			.bandwidth = 40,
			.num_channel = 6,
			.channel = {
				{ .channel = 104, .ctrl_channels = {104}},
				{ .channel = 112, .ctrl_channels = {112}},
				{ .channel = 120, .ctrl_channels = {120}},
				{ .channel = 128, .ctrl_channels = {128}},
				{ .channel = 136, .ctrl_channels = {136}},
				{ .channel = 144, .ctrl_channels = {144}},
			}
		},
		{
			.id = 124,
			.bandwidth = 40,
			.num_channel = 4,
			.channel = {
				{ .channel = 149, .ctrl_channels = {149}},
				{ .channel = 153, .ctrl_channels = {153}},
				{ .channel = 157, .ctrl_channels = {157}},
				{ .channel = 161, .ctrl_channels = {161}},
			}
		},
		{
			.id = 125,
			.bandwidth = 20,
			.num_channel = 6,
			.channel = {
				{ .channel = 149, .ctrl_channels = {149}},
				{ .channel = 153, .ctrl_channels = {153}},
				{ .channel = 157, .ctrl_channels = {157}},
				{ .channel = 161, .ctrl_channels = {161}},
				{ .channel = 164, .ctrl_channels = {164}},
				{ .channel = 169, .ctrl_channels = {169}},
			}
		},
		{
			.id = 126,
			.bandwidth = 40,
			.num_channel = 2,
			.channel = {
				{ .channel = 149, .ctrl_channels = {149}},
				{ .channel = 157, .ctrl_channels = {157}},
			}
		},
		{
			.id = 127,
			.bandwidth = 40,
			.num_channel = 2,
			.channel = {
				{ .channel = 153, .ctrl_channels = {153}},
				{ .channel = 161, .ctrl_channels = {161}},
			}
		},
		{
			.id = 128,
			.bandwidth = 80,
			.num_channel = 6,
			.channel = {
				{ .channel = 42, .ctrl_channels = {36, 40, 44, 48}},
				{ .channel = 58, .ctrl_channels = {52, 56, 60, 64}},
				{ .channel = 106, .ctrl_channels = {100, 104, 108, 112}},
				{ .channel = 122, .ctrl_channels = {116, 120, 124, 128}},
				{ .channel = 138, .ctrl_channels = {132, 136, 140, 144}},
				{ .channel = 155, .ctrl_channels = {149, 153, 157, 161}},
			}
		},
		{
			.id = 129,
			.bandwidth = 160,
			.num_channel = 2,
			.channel = {
				{ .channel = 50, .ctrl_channels = {36, 40, 44, 48, 52, 56, 60, 64}},
				{ .channel = 114, .ctrl_channels = {100, 104, 108, 112, 116, 120, 124, 128}},
			}
		},
		{
			.id = 130,
			.bandwidth = 80,
			.num_channel = 6,
			.channel = {
				{ .channel = 42, .ctrl_channels = {36, 40, 44, 48}},
				{ .channel = 58, .ctrl_channels = {52, 56, 60, 64}},
				{ .channel = 106, .ctrl_channels = {100, 104, 108, 112}},
				{ .channel = 122, .ctrl_channels = {116, 120, 124, 128}},
				{ .channel = 138, .ctrl_channels = {132, 136, 140, 144}},
				{ .channel = 155, .ctrl_channels = {149, 153, 157, 161}},
			}
		},
	},
};

const struct wifi_radio_opclass *wifi_opclass_e4(struct wifi_radio_opclass *dst)
{
	if (dst)
		memcpy(dst, &e4, sizeof(e4));

	return &e4;
}

struct wifi_radio_opclass_entry *wifi_opclass_find_entry(struct wifi_radio_opclass *opclass, uint8_t id)
{
	struct wifi_radio_opclass_entry *entry;
	int i;

	for (i = 0; i < opclass->num_opclass; i++) {
		entry = &opclass->entry[i];
		if (entry->id != id)
			continue;

		return entry;
	}

	return NULL;
}

int wifi_get_opclass_entry(uint32_t opclass, struct wifi_radio_opclass_entry *e)
{
	return -1;
#if 0

	struct wifi_opclass_entry *tab;
	struct wifi_opclass_entry *ptr;
	int tabsize;
	int i;

	tab = (struct wifi_opclass_entry *)wifi_opclass_global;
	tabsize = wifi_opclass_global_size;

	for (i = 0; i < tabsize; i++) {
		ptr = tab + i;
		if (ptr->opclass == opclass) {
			memcpy(e, ptr, sizeof(struct wifi_opclass_entry));
			return 0;
		}
	}

	return -1;
#endif
}

